@@ -19,20 +19,46 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <libbonobo.h>
-#include <gnome-speech/gnome-speech.h>
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include <libbonobo.h>
+#include <gnome-speech/gnome-speech.h>
+
 #include "spgs.h"
 #include "spgscb.h"
 #include "libsrconf.h"
 #include "SRMessages.h"
 
-#define SPGS_DEBUG
+
 #undef SPGS_DEBUG
+#undef SPGS_DEBUG_CB
+
+typedef enum
+{
+    GS_ERR_GENERIC,
+    GS_ERR_SERVER_ACTIVATION,
+    GS_ERR_DRIVER_NAME_INVALID,
+    GS_ERR_NO_VOICES,
+    GS_ERR_VOICE_CREATE,
+    GS_ERR_VOICE_INVALID,
+    GS_ERR_BONOBO_DUPLICATE,
+    GS_ERR_LAST_DEFINED
+} GSErrorType;
+
+#define GS_ERR_NO  GS_ERR_LAST_DEFINED
+
+char *gs_errors [GS_ERR_NO] =
+{
+    "Generic error",
+    "Activation of \"gnome-speech\" server failed.",
+    "Failed to get the driver name from server",
+    "Current driver has no voices",
+    "Could NOT create speaker",
+    "Invalid voice",
+    "Failed to duplicate CORBA Object and reference it"
+};
 /*_________________________________< GLOBALS >________________________________*/
 
 typedef struct 
@@ -40,6 +66,7 @@ typedef struct
     float 	min;
     float	max;
 } GSMinMaxType;
+
 typedef enum
 {
     GS_PARAM_RATE,
@@ -60,28 +87,70 @@ typedef struct
     GSMinMaxType			volume;
 } GSSpeaker;
 
+typedef GHashTable GSSpeakers;
+/*_________________________________</GLOBALS >________________________________*/
 TTSEngine 				*tts_engine;
 
-static gint 				driver_cnt 	= 0;
-static gint 				voice_cnt	= 0;
-
-static GHashTable  			*gs_speakers_hash_table;
+static GSSpeakers  			*gs_speakers_hash_table;
 static GSSpeaker  			*default_speaker;
-static GNOME_Speech_Speaker 		current_speaker;
-/*_________________________________</GLOBALS >________________________________*/
+static GNOME_Speech_Speaker 		current_speaker = CORBA_OBJECT_NIL;
+
+static CORBA_Environment		ev = { 0 };
+
 
 /*___________________________________< AUX >__________________________________*/
-void
-check_return_value (CORBA_Environment 	*env,
-		    int 		line)
+/**
+ * gs_ev:
+ *
+ * Initialize the exception handling mechanism of CORBA.
+ *
+ * Returns: Exception's data in a #CORBA_Environment structure.
+ **/
+CORBA_Environment *
+gs_ev (void)
 {
-/*FIXME*/
-    if ( (env) && (env)->_major != CORBA_NO_EXCEPTION)
+    CORBA_exception_init (&ev);
+    return &ev;
+}
+
+/**
+ * gs_peek_ev:
+ *
+ * Returns: Exception's data in a #CORBA_Environment structure.
+ **/
+
+CORBA_Environment *
+gs_peek_ev (void)
+{
+    return &ev;
+}
+/**
+ * gs_check_ev:
+ *
+ * Returns: #FALSE if there was an exception, #TRUE otherwise.
+ **/
+gboolean
+gs_check_ev (GSErrorType	error_number,
+	     int 		line)
+{
+    CORBA_Environment *ev = gs_peek_ev ();
+    
+    if ( ev && ev->_major != CORBA_NO_EXCEPTION)
     {
-	sru_warning ("Exception \"%s\" occured at line %d.",
-		    bonobo_exception_get_text (env), line);
-	CORBA_exception_free (env);
+	char *error;
+	
+	error = bonobo_exception_get_text (ev);
+
+	sru_warning ("Exception \"%s\" [\"%s\"] occured at line %d.",
+		    gs_errors[error_number], error, line);
+
+	g_free (error);
+	
+	CORBA_exception_free (ev);
+	
+	return FALSE;
     }
+    return TRUE;
 }
 /*___________________________________</AUX >__________________________________*/
 
@@ -94,19 +163,26 @@ gs_callback (gint 	reason,
     switch (reason)
     {
 	 case GNOME_Speech_speech_callback_speech_started:
-#ifdef  SPGS_DEBUG
+#ifdef  SPGS_DEBUG_CB
 	    sru_message ("gs_callback - event started"); 
 #endif
     	    tts_engine->callback (TTS_EV_SPEECH_STARTED, NULL, client_data);
 	    break;
 
 	case GNOME_Speech_speech_callback_speech_ended:
-#ifdef  SPGS_DEBUG
-	    sru_message ("gs_callback - event ended\n");
+#ifdef  SPGS_DEBUG_CB
+	    sru_message ("gs_callback - event ended"); 
 #endif
     	    tts_engine->callback (TTS_EV_END_OF_SPEECH, NULL, client_data);
 	    break;
 
+	case GNOME_Speech_speech_callback_speech_progress:
+#ifdef  SPGS_DEBUG_CB
+	    sru_message ("gs_callback - event progress"); 
+#endif
+    	    tts_engine->callback (TTS_EV_WORD_MARKER, NULL, client_data);
+	    break;
+
 	default:
     	    sru_warning ("gs_callback - unknown event\n");
     	    break;
@@ -119,25 +195,23 @@ GSSpeaker *
 gs_speaker_new0 (GNOME_Speech_SynthesisDriver	driver,
 		 gint                          	voice_idx)
 {
-/*FIXME*/
-    GSSpeaker *gss = NULL;
-    CORBA_Environment 		ev;
+    GSSpeaker	*gss = NULL;
 
     gss = g_new0 (GSSpeaker, 1);
 
     gss->driver 	= CORBA_OBJECT_NIL;
     gss->speaker	= CORBA_OBJECT_NIL;
-
+    gss->callback 	= NULL;
+    
     if (driver)
     {
 	/* store the driver */
-	CORBA_exception_init (&ev);
 /*        
-	GNOME_Speech_SynthesisDriver_ref (driver, &ev);
+	GNOME_Speech_SynthesisDriver_ref (driver, gs_ev ());
 	gss->driver = driver;
 */	
-        gss->driver = bonobo_object_dup_ref (driver, &ev); 
-	check_return_value (&ev, __LINE__);
+        gss->driver = bonobo_object_dup_ref (driver, gs_ev () ); 
+	gs_check_ev (GS_ERR_GENERIC, __LINE__);
     }
     
      /* store the voice index in driver */
@@ -149,42 +223,58 @@ gs_speaker_new0 (GNOME_Speech_SynthesisDriver	driver,
 void
 gs_speaker_free (GSSpeaker *gss)
 {
-/*FIXME*/
-    CORBA_Environment 		ev;
-    
     if (gss)
     {
 #ifdef SPGS_DEBUG
-	fprintf (stderr,"\ngs_speaker_free : voice_idx = %d",
+	sru_debug ("gs_speaker_free : voice_idx = %d",
 			    gss->voice_idx);
 #endif
-/*
-	    GNOME_Speech_SpeechCallback_unref (gss->callback, NULL);
-*/
+	if (gss->callback != CORBA_OBJECT_NIL)
 	    bonobo_object_unref (gss->callback);
 
 	if (gss->driver != CORBA_OBJECT_NIL)
 	{
-	    CORBA_exception_init (&ev);
 /*
 	    GNOME_Speech_SynthesisDriver_unref(gss->driver, NULL);
 */
-	    bonobo_object_release_unref (gss->driver, &ev);
-	    check_return_value (&ev, __LINE__);
+	    bonobo_object_release_unref (gss->driver, gs_ev () );
+	    gs_check_ev (GS_ERR_GENERIC, __LINE__);
 	    
 	}
 	
 	if (gss->speaker != CORBA_OBJECT_NIL)
 	{
-	    CORBA_exception_init (&ev);
-	    GNOME_Speech_Speaker_unref(gss->speaker, NULL);
-	    check_return_value (&ev, __LINE__);
+	    GNOME_Speech_Speaker_unref(gss->speaker, gs_ev () );
+	    gs_check_ev (GS_ERR_GENERIC, __LINE__);
 	}
 	g_free (gss);
 	gss = NULL;
     }
 }
 
+
+void
+gs_speaker_debug (GSSpeaker	*speaker)
+{
+#ifdef SPGS_DEBUG
+    if (speaker)
+    {
+	sru_debug ("_____________________________");
+	sru_debug ("speaker             %p", speaker);
+	sru_debug ("speaker->driver     %p", speaker->driver);
+	sru_debug ("speaker->voice_idx  %d", speaker->voice_idx);
+	sru_debug ("speaker->speaker    %p", speaker->speaker);
+	sru_debug ("speaker->callback   %p", speaker->callback);
+	sru_debug ("speaker->cb_support %d", speaker->cb_support);
+
+	sru_debug ("default_speaker     %p", default_speaker);
+	sru_debug ("current_speaker     %p", current_speaker);
+	sru_debug ("_____________________________");
+    }
+#endif
+}
+
+
 float
 gs_speaker_procent_to_units (GSSpeaker	*speaker,
 			    GSParameter	parameter,
@@ -221,7 +311,7 @@ gs_speaker_procent_to_units (GSSpeaker	*speaker,
 	    break;    
     }
 #ifdef SPGS_DEBUG
-    sru_message ("\n%d. procents %f , units %f",
+    sru_debug ("%d. procents %f , units %f",
 		    parameter,
 		    procent,
 		    units);
@@ -241,6 +331,7 @@ gs_normilize_driver_name (gchar *driver_name)
     
     return new_driver_name;
 }
+
 void
 gs_speaker_to_gconf (GSList *driver_voice_list, 
 		    const gchar *driver_name)
@@ -315,7 +406,7 @@ gs_speakers_get_speaker (gchar 	*key)
     GSSpeaker 	*speaker = NULL;
 
 #ifdef  SPGS_DEBUG
-    sru_message ("gs_speakers_get_speaker key : %s", key);
+    sru_debug ("gs_speakers_get_speaker key : %s", key);
 #endif
 
     if (key)
@@ -343,17 +434,14 @@ gs_speakers_add_speaker (GSSpeaker 	*speaker,
 GSSpeaker *
 gs_speakers_select_speaker (gchar *voice_name)
 {
-    CORBA_Environment 		ev;
     BonoboObject 		*bo;
     GNOME_Speech_VoiceInfoList 	*voices;
     GNOME_Speech_ParameterList	*parameters;
     
-    GSSpeaker 			*gss = NULL;
+    GSSpeaker 			*gss;
     Callback 			*cb;
     gint			param_n;
 
-    CORBA_exception_init (&ev);
-
     if (!voice_name)
     {
 	/* default speaker request */
@@ -373,53 +461,75 @@ gs_speakers_select_speaker (gchar *voice_name)
 	}
     }
     /* Instantiation on demand - 
-	do I already instantiated a speaker for this voice
-    (!!! TBR !!! could change the params here too ) */
+	do I already instantiated a speaker for this voice */
 
     if (gss && !gss->speaker)
     {
 	/* create a speaker with this voice */
 	voices = GNOME_Speech_SynthesisDriver_getAllVoices (gss->driver, 
-							    &ev);
+							    gs_ev () );
+	if (!gs_check_ev (GS_ERR_GENERIC, __LINE__))
+	    return default_speaker;
 
 	if (voices)
 	{
-    	    gss->speaker = GNOME_Speech_SynthesisDriver_createSpeaker (
+	    gss->speaker = GNOME_Speech_SynthesisDriver_createSpeaker (
 			    gss->driver,
         		    &voices->_buffer[gss->voice_idx], 
-			    &ev);
-	    check_return_value (&ev, __LINE__);
+			    gs_ev () );
+
+		if (BONOBO_EX (&ev) )
+		{
+		    gs_check_ev (GS_ERR_GENERIC, __LINE__);
+		    sru_warning ("gs_select_speaker: Could not create speaker for %s voice name.", voice_name);
+		    return  default_speaker;
+		}
+
 
 	    if (gss->speaker)
 	    {
 		parameters = GNOME_Speech_Speaker_getSupportedParameters (gss->speaker,
-									&ev);
-		check_return_value (&ev, __LINE__);
+									  gs_ev () );
+		if (BONOBO_EX (&ev) || parameters->_length == 0)
+		{
+		    gs_check_ev (GS_ERR_GENERIC, __LINE__);
+
+		    sru_warning ("gs_select_speaker: No parameters available for %s. Speaker is not valid.", voice_name);
+
+		    return  default_speaker;
+		}
+#ifdef  SPGS_DEBUG		    		
+		sru_debug ("gs_select_speaker: Parameters available for %s : %d. ", voice_name, parameters->_length);
+#endif
 		/* get min and max values for the parameters in wich we are interestead*/
 		for ( param_n = 0; param_n < parameters->_length ; param_n ++)
 		{
 		    GNOME_Speech_Parameter *param = &(parameters->_buffer[param_n]);
 		    
 		    if (!param)
-			continue;
+		    {
+			GNOME_Speech_Speaker_unref(gss->speaker, NULL);
+			gss->speaker = CORBA_OBJECT_NIL;
+			return default_speaker;
+		    }
 		    if (!strcmp (param->name, "rate") ) 
 		    {
 			gss->rate.min = param->min;
 			gss->rate.max = param->max;
 #ifdef SPGS_DEBUG
-			sru_message ("param %s min %.2lf max %.2lf current %.2lf",
+			sru_debug ("param %s min %.2lf max %.2lf current %.2lf",
 					param->name,
 					gss->rate.min,
 					param->max,
 					param->current);  
 #endif
 		    }
-		    if (!strcmp (param->name, "pitch fluctuation") ) 
+		    if (!strcmp (param->name, "pitch") ) 
 		    {
 			gss->pitch.min = param->min;
 			gss->pitch.max = param->max;
 #ifdef SPGS_DEBUG
-			sru_message ("param %s min %.2lf max %.2lf current %.2lf",
+			sru_debug ("param %s min %.2lf max %.2lf current %.2lf",
 					param->name,
 					param->min,
 					param->max,
@@ -431,7 +541,7 @@ gs_speakers_select_speaker (gchar *voice_name)
 			gss->volume.min = param->min;
 			gss->volume.max = param->max;
 #ifdef SPGS_DEBUG		    
-			sru_message ("param %s min %.2lf max %.2lf current %.2lf",
+			sru_debug ("param %s min %.2lf max %.2lf current %.2lf",
 					param->name,
 					param->min,
 					param->max,
@@ -441,32 +551,39 @@ gs_speakers_select_speaker (gchar *voice_name)
 		}
 /*FIXME*/		
     		/* register a callback for this speaker */
-    		cb = callback_new (gs_callback, NULL);
-		bo = BONOBO_OBJECT (cb);
-
-    		gss->callback = bo;
-	
-		GNOME_Speech_Speaker_registerSpeechCallback (gss->speaker,
-							bonobo_object_corba_objref (bo),
-							&ev);
-		if ( (&ev) && (&ev)->_major != CORBA_NO_EXCEPTION)
+    		if (!gss->callback)
 		{
-		    sru_message ("Callbacks are NOT supported by all drivers");
+		    gboolean rv = FALSE;
 		    
-		    bonobo_object_unref (gss->callback);
-		    
-		    gss->callback = NULL;
-		    gss->cb_support = FALSE;
-		}
-		else
-		    gss->cb_support = TRUE;
+		    cb = callback_new (gs_callback, NULL);
+		    bo = BONOBO_OBJECT (cb);
+
+    		    gss->callback = bo;
+	
+		    rv = GNOME_Speech_Speaker_registerSpeechCallback (gss->speaker,
+							bonobo_object_corba_objref (bo),
+							gs_ev () );
+		    if (!rv || !gs_check_ev (GS_ERR_GENERIC, __LINE__) )
+		    {
+			if (voice_name)
+			{
+			    sru_warning ("Callbacks are NOT supported by \"%s\" voice.", voice_name);
+			}
+			bonobo_object_unref (gss->callback);
 		    
+			gss->callback = NULL;
+			gss->cb_support = FALSE;
+		    }
+		    else
+			gss->cb_support = TRUE;
+		}    
 		CORBA_free (parameters);
 	    }
 	    CORBA_free (voices);
 	}
     }
     /* !!! TBR !!! should have some sanity check and error reporting here */
+    gs_speaker_debug (gss);
     return gss;
 }
 
@@ -495,6 +612,7 @@ gs_speak (SRSVoice 	*voice,
 	  SRSText 	*text )
 {
     GSSpeaker *gss;
+
     /* sel current speaker for voice->TTSVoiceName */
     gss = gs_speakers_select_speaker (voice->tts_voice_name);
     if (gss)
@@ -503,17 +621,10 @@ gs_speak (SRSVoice 	*voice,
     
 	if (!gss->cb_support )
 	{
-	    if (voice->cb_support == -1)
-    		sru_warning ("Voice %s is NOT supporting speech markers",
-			voice->tts_voice_name);
 	    voice->cb_support = 0;
 	}
 	else
 	{
-/*	
-	    sru_message ("Voice %s is supporting speech markers",
-			voice->tts_voice_name);
-*/
 	    voice->cb_support = 1;
 	}
     }
@@ -522,7 +633,6 @@ gs_speak (SRSVoice 	*voice,
 	CORBA_Environment 	ev;
 	float 			val;
 	    
-	CORBA_exception_init (&ev);
 
 	val = gs_speaker_procent_to_units (gss,
 					  GS_PARAM_RATE,
@@ -530,8 +640,8 @@ gs_speak (SRSVoice 	*voice,
 	GNOME_Speech_Speaker_setParameterValue (current_speaker, 
 						"rate", 
 						val, 
-						&ev);
-	check_return_value (&ev, __LINE__);						
+						gs_ev () );
+	gs_check_ev (GS_ERR_GENERIC, __LINE__);						
 
 	val = gs_speaker_procent_to_units (gss,
 					  GS_PARAM_VOLUME,
@@ -539,31 +649,37 @@ gs_speak (SRSVoice 	*voice,
 	GNOME_Speech_Speaker_setParameterValue (current_speaker, 
 						"volume", 
 						val, 
-						&ev);
-	check_return_value (&ev, __LINE__);
+						gs_ev () );
+	gs_check_ev (GS_ERR_GENERIC, __LINE__);
 	
 	val = gs_speaker_procent_to_units (gss,
 					  GS_PARAM_PITCH,	
 					  (float)voice->pitch);
 	GNOME_Speech_Speaker_setParameterValue (current_speaker, 
-						"pitch fluctuation", 
+						"pitch", 
 						val, 
-						&ev);
-	check_return_value (&ev, __LINE__);
+						gs_ev () );
+	gs_check_ev (GS_ERR_GENERIC, __LINE__);
 
         if (text && text->text) 
 	{
+
 #ifdef SPGS_DEBUG
-	sru_message ("(%s %d) : gs_speak \"%s\"",
+	sru_debug ("(%s %d) : gs_speak \"%s\" | current_speaker %p",
 		    __FILE__,
 		    __LINE__,
-		    text->text);
+		    text->text,
+		    current_speaker);
 #endif
     	    /* resume_text = text->Text; */
     	    GNOME_Speech_Speaker_say (current_speaker, 
 				      text->text, 
-				      &ev);
-	    check_return_value (&ev, __LINE__);
+				      gs_ev () );
+
+	    if (BONOBO_EX (&ev) )
+	    {
+		gs_check_ev (GS_ERR_GENERIC, __LINE__);
+	    }
         }
     }	
 }
@@ -571,62 +687,56 @@ gs_speak (SRSVoice 	*voice,
 void
 gs_shut_up (void)
 {
-    CORBA_Environment ev;
-    CORBA_exception_init (&ev);
-    
     if (current_speaker)
     {
+#ifdef SPGS_DEBUG
+	sru_debug ("(%s %d) : gs_shut_up |  current_speaker %p",
+		    __FILE__,
+		    __LINE__,
+		    current_speaker);
+#endif
+
 	GNOME_Speech_Speaker_stop (current_speaker, 
-				   &ev);
-    	check_return_value (&ev, __LINE__);
+				   gs_ev () );
+    	gs_check_ev (GS_ERR_GENERIC, __LINE__);
     }
 }
 
 void gs_pause ()
 {
-    CORBA_Environment ev;
-    
-    CORBA_exception_init (&ev);
-
     /* WAITING FOR SUPPORT FROM GS */
-    check_return_value (&ev, __LINE__);
+    gs_check_ev (GS_ERR_GENERIC, __LINE__);
 }
 
 void gs_resume ()
 {
-    CORBA_Environment ev;
-    
-    CORBA_exception_init (&ev);
-
     /* WAITING FOR SUPPORT FROM GS */
 
-    check_return_value (&ev, __LINE__);
+    gs_check_ev (GS_ERR_GENERIC, __LINE__);
 }
 
 
 Bonobo_ServerInfoList *
 gs_init_get_gs_servers (void)
 {
-    Bonobo_ServerInfoList 	*servers;
-    CORBA_Environment 		ev;
-
-
-    CORBA_exception_init (&ev);
-
+    Bonobo_ServerInfoList 	*servers  = CORBA_OBJECT_NIL;
+/*
+    Bonobo_ServerInfoList 	*callback_servers =  CORBA_OBJECT_NIL;
+*/
     if (!bonobo_init (NULL, NULL) )
     { 
 	/*this is failure*/
 	sru_warning ("Bonobo initialization failed.");
 	return NULL;
     }
+    
     servers = bonobo_activation_query (
 		"repo_ids.has ('IDL:GNOME/Speech/SynthesisDriver:0.2')",
 		NULL, 
-		&ev);
+		gs_ev () );
 
-    if (BONOBO_EX (&ev)) 
+    if (!gs_check_ev (GS_ERR_SERVER_ACTIVATION, __LINE__) ) 
     {
-	sru_warning ("Activation of \"gnome-speech\" server failed.");
 	return NULL;
     }
     
@@ -635,7 +745,19 @@ gs_init_get_gs_servers (void)
 	sru_warning ("No \"gnome-speech\" drivers were found.");
 	return NULL;
     }
-
+/*
+    callback_servers = bonobo_activation_query (
+		"repo_ids.has ('IDL:GNOME/Speech/SpeechCallback:0.2')",
+		NULL, 
+		gs_ev () );
+    if (!gs_check_ev (GS_ERR_SERVER_ACTIVATION, __LINE__) ||
+	!callback_servers) 
+    {
+	sru_warning ("No \"gnome-speech\" drivers supports callbacks.");
+    }
+    else
+	sru_message ("Some/all \"gnome-speech\" drivers supports callbacks.");
+*/
     return servers;
 }
 
@@ -645,31 +767,29 @@ CORBA_Object
 gs_init_activate_server (Bonobo_ServerInfo 	*info)
 {
 	CORBA_Object		obj = CORBA_OBJECT_NIL;
-	CORBA_Environment	ev;
 
 	/* atempt to activate the server */
 
-	CORBA_exception_init (&ev);
 	/*obj = a CORBAobject reference to the newly activated server*/
 	obj = bonobo_activation_activate_from_id (info->iid, 
 						  0, 
 						  NULL, 
-						  &ev);
+						  gs_ev () );
 	
 	/*check the environment for failure*/
 	if (BONOBO_EX (&ev) || obj == CORBA_OBJECT_NIL) 
 	{
 #ifdef SPGS_DEBUG	
-	    sru_warning ("Activation of server %s failed.\n", info->iid);
+	    sru_warning ("Activation of server %s failed.", info->iid);
 #endif
 	}
 	else
 	{
-	    driver_cnt++;
+	    if (!GNOME_Speech_SynthesisDriver_driverInit (obj, &ev))
+			return CORBA_OBJECT_NIL;
 #ifdef SPGS_DEBUG
-	    sru_message ("Activation of server %s succedded. Total of %d successes", 
-			 info->iid,
-			 driver_cnt);
+	    sru_message ("Activation of server %s succedded.", 
+			 info->iid);
 #endif
 	}
 	return  obj;
@@ -679,10 +799,11 @@ int
 gs_init (TTSEngine *engine)
 {
     gint 			server_n = 0, 
-				voice_n	 = 0;
-    gchar 			*voice_name;
+				voice_n	 = 0,
+				voice_cnt = 0;
+    gchar 			*voice_name = NULL,
+				*default_voice_name = NULL;
 
-    CORBA_Environment 		ev;
     CORBA_string 		driver_name;/*, driver_version;*/
     Bonobo_ServerInfoList 	*servers;
     GSSpeaker			*gss;
@@ -690,7 +811,10 @@ gs_init (TTSEngine *engine)
     GSList			*speech_driver_list = NULL;
     GSList			*driver_voice_list  = NULL;
     GSList			*tmp;
-/*FIXME*/
+
+    default_speaker = NULL;
+    current_speaker = CORBA_OBJECT_NIL;
+    
     /*create internal hash table*/
     gs_speakers_init();
 	
@@ -700,7 +824,6 @@ gs_init (TTSEngine *engine)
 	return FALSE;
 	     
     voice_cnt 	= 0;
-    driver_cnt 	= 0;	
     
     for (server_n = 0; server_n < servers->_length; server_n++)
     {		
@@ -709,9 +832,6 @@ gs_init (TTSEngine *engine)
 	GNOME_Speech_SynthesisDriver 	driver;
 	GSSpeaker 			*gss;
 
-
-	CORBA_exception_init (&ev);
-		
 	info = &servers->_buffer[server_n];
 	driver = (GNOME_Speech_SynthesisDriver) gs_init_activate_server (info);
 	
@@ -719,28 +839,42 @@ gs_init (TTSEngine *engine)
 	    continue; /* try next server*/
 
 	driver_name = GNOME_Speech_SynthesisDriver__get_driverName (driver, 
-								    &ev);
-	check_return_value (&ev, __LINE__);
-
-	speech_driver_list = g_slist_append (speech_driver_list, 
-					     g_strdup (gs_normilize_driver_name (driver_name)));
+								    gs_ev () );
+	if (!gs_check_ev (GS_ERR_DRIVER_NAME_INVALID, __LINE__) )
+	{
+	    CORBA_free (driver_name);
+	    continue; /* try next server*/
+	}
 
 	/* create a list of available voices in gconf */
 	voices = GNOME_Speech_SynthesisDriver_getAllVoices (driver, 
-							    &ev);
-	if (BONOBO_EX (&ev) 	|| 
-	    !voices 		|| 
+							    gs_ev () );
+	if (!gs_check_ev (GS_ERR_NO_VOICES, __LINE__) 	|| 
+	    !voices 					|| 
 	    !voices->_length) 
-	{
+    	{
 	    sru_warning ("Driver named \"%s\" has no available voices.", 
 			 driver_name);
 	    continue; /* try next server*/
 	}
-		
-	for (voice_n = 0; voice_n < voices->_length  && driver_name; ++voice_n) 
+
+	
+        current_speaker =  GNOME_Speech_SynthesisDriver_createSpeaker (
+			    driver,
+        		    &voices->_buffer[0],
+			    gs_ev () );
+        if (!gs_check_ev (GS_ERR_VOICE_CREATE, __LINE__))
+	{
+	    sru_warning ("Could not create speaker for \"V0 %s - %s\" voice name.", voices->_buffer[0].name, driver_name );
+	    continue;
+	}
+	for (voice_n = 0; voice_n < voices->_length; ++voice_n) 
 	{
 	    /* store the driver together with the voice index to allow a speaker creation later */
-    	    gss = gs_speaker_new0 (driver, voice_n);
+	    GNOME_Speech_Speaker_setParameterValue (current_speaker, "voice", voice_n+1, gs_ev ());
+	    if (!gs_check_ev (GS_ERR_VOICE_INVALID, __LINE__) )
+		break;
+	    gss = gs_speaker_new0 (driver, voice_n);
     	    
 	    voice_name 	= g_strdup_printf ("V%d %s - %s", 
 					    voice_n, 
@@ -750,27 +884,42 @@ gs_init (TTSEngine *engine)
 						g_strdup (voice_name));
 
 	    voice_cnt++;
-    	    gs_speakers_add_speaker (gss, voice_name);
 
     	    /* select a default voice */
-    	    if (server_n <= 1 && voice_n == 0) 
-		default_speaker = gss; /* !!!!!!! MEGA HACK !!!!!!!!!!!*/
+    	    if (!default_speaker) 
+	    {
+		default_speaker = gss; 
+#ifdef SRSP_DEBUG
+		sru_message ("gs_init: DEFAULT voice_n %d, voice_cnt %d, voice_name %s ",voice_n, voice_cnt, voice_name);
+#endif
+		default_voice_name = g_strdup (voice_name);		
+	    }
+    	    gs_speakers_add_speaker (gss, voice_name);
 
 #ifdef SRSP_DEBUG
-	    sru_message("gs_init : Driver \"%s\" voice_name %s",
+	    sru_debug("gs_init: Driver \"%s\" voice_name %s",
 			driver_name,
 			voice_name);
 #endif
     	    g_free (voice_name);
 	}/*end of voices*/
-	
-        gs_speaker_to_gconf (driver_voice_list, driver_name);
-	for (tmp = driver_voice_list; tmp ; tmp = tmp->next)
-	    g_free (tmp->data);
-	g_slist_free (driver_voice_list);
-	
-	driver_voice_list = NULL;
+	bonobo_object_release_unref (current_speaker, gs_ev () );
+        
+	if (driver_voice_list)
+	{
+#ifdef SRSP_DEBUG
+	    sru_debug("gs_init: Driver \"%s\" has available voices",driver_name);
+#endif
+	    gs_speaker_to_gconf (driver_voice_list, driver_name);
+	    for (tmp = driver_voice_list; tmp ; tmp = tmp->next)
+		g_free (tmp->data);
+	    g_slist_free (driver_voice_list);
+	    driver_voice_list = NULL;
+	    
+	    speech_driver_list = g_slist_append (speech_driver_list, 
+					     g_strdup (gs_normilize_driver_name (driver_name) ) );
 
+	}
 	CORBA_free (voices);
 	CORBA_free (driver_name);	
 	CORBA_Object_release (driver, NULL);
@@ -787,11 +936,12 @@ gs_init (TTSEngine *engine)
     CORBA_free (servers);
 
     /* select a default speaker */
-    gss = gs_speakers_select_speaker (NULL);
+    gss = gs_speakers_select_speaker (default_voice_name);
     if (gss)
 	current_speaker = gss->speaker;
+    g_free (default_voice_name);
     tts_engine = engine;
-    if (driver_cnt)
+    if (voice_cnt)
     {		
 	/* fill the engine structure with the actual functions */
    	tts_engine->shut_up	= gs_shut_up;
@@ -801,7 +951,7 @@ gs_init (TTSEngine *engine)
 	tts_engine->terminate	= gs_terminate;
     }
     
-    return (driver_cnt > 0) ? TRUE : FALSE;
+    return (voice_cnt > 0) ? TRUE : FALSE;
 }
 
 void
@@ -809,5 +959,6 @@ gs_terminate (void)
 {
     gs_shut_up ();
     gs_speakers_terminate();
+    bonobo_debug_shutdown ();
 }
 /*______________________________</GS API >____________________________________*/
