@@ -36,9 +36,8 @@
 #include "srmain.h"
 #include "srpres.h"
 #include "srintl.h"
+#include "screen-review.h"
 
-#define MIN_BRAILLE_PORT 1
-#define MAX_BRAILLE_PORT 4
 
 static SRLClientHandle src_srl_client;
 gboolean src_use_braille = FALSE;
@@ -50,6 +49,9 @@ extern gchar *src_braille_device;
 extern gint src_braille_port;
 extern gint src_braille_position_sensor;
 extern gint src_braille_optical_sensor;
+extern gchar *src_braille_style;
+extern gchar *src_braille_cursor_style;
+extern gchar *src_braille_translation_table;
 extern gint src_speech_mode;
 gboolean src_mouse_take;
 gboolean src_mouse_click;
@@ -127,7 +129,7 @@ src_present_real_sro (SRObject *obj)
 		gchar status_[5] = "    ";
 
 		if (sro_text_get_caret_offset (obj, &offset, SR_INDEX_CONTAINER))
-		    cursor = g_strdup_printf (" cursor=\"%ld\" cstyle=\"underline\"", offset + 4);
+		    cursor = g_strdup_printf (" cursor=\"%ld\" cstyle=\"%s\"", offset + 4, src_braille_cursor_style);
 	    
 		if (sro_text_get_text_attr_from_caret (obj, SR_TEXT_BOUNDARY_CHAR, 
 				    &attr, SR_INDEX_CONTAINER))
@@ -167,7 +169,7 @@ src_present_real_sro (SRObject *obj)
 		}
 	    }
 	    sprintf (offset, "%d", src_braille_offset);
-	    tmp = g_strconcat ("<BRLOUT language=\"us\" bstyle=\"8\" clear=\"true\">",
+	    tmp = g_strconcat ("<BRLOUT language=\"",src_braille_translation_table,"\" bstyle=\"", src_braille_style,"\" clear=\"true\">",
 				"<BRLDISP role=\"main\" offset=\"", offset, "\" clear=\"true\"",
 				cursor ? cursor : "",
 				">",
@@ -573,18 +575,18 @@ src_process_config_changed_for_braille (SRConfigStructure *config)
 		if (src_braille_device) 
 		    g_free (src_braille_device);
 		
-		src_braille_device = g_strdup (config->newvalue);			
+		src_braille_device = g_strdup (config->newvalue);
 	    
 		restart = TRUE;
 	    }
 	    else
 	    {
-		fprintf (stderr, "SR: process config changed for braille: brl_device did not change!!!\n");
+		sru_message ("SR: process config changed for braille: brl_device did not change!!!");
 	    }
 	}
 	else
 	{
-	    fprintf (stderr, "SR: process config changed for braille: brl_device NULL!!!\n");
+	    sru_message ("SR: process config changed for braille: brl_device NULL!!!");
 	}
     }
     else
@@ -606,18 +608,51 @@ src_process_config_changed_for_braille (SRConfigStructure *config)
     {
     }
     else
-    if (config->key && strcmp (config->key, BRAILLE_STYLE) == 0)
+    */
+    if (config->key && 
+	config->type == CFGT_STRING && 
+	strcmp (config->key, BRAILLE_STYLE) == 0)
     {
+	if (config->newvalue)
+	{
+	    src_braille_style = g_strdup ((gchar*)config->newvalue);
+	}
     }
     else
-    if (config->key && strcmp (config->key, BRAILLE_CURSOR) == 0)
+    
+    if (config->key && 
+	config->type == CFGT_STRING && 
+	strcmp (config->key, BRAILLE_CURSOR_STYLE) == 0)
     {
+    	if (config->newvalue)
+	{
+	    g_free (src_braille_cursor_style);
+	    src_braille_cursor_style = g_strdup (config->newvalue);			
+	}
     }
     else
-    if (config->key && strcmp (config->key, BRAILLE_TRANSLATION) == 0)
+    if (config->key && 
+	config->type == CFGT_STRING && 
+	strcmp (config->key, BRAILLE_TRANSLATION) == 0)
     {
+    	if (config->newvalue)
+	{
+	    if (!strcmp (config->newvalue, src_braille_translation_table))
+		return;
+	    g_free (src_braille_translation_table);
+		
+	    src_braille_translation_table = g_strdup (config->newvalue);				    
+	    
+	    restart = TRUE;
+	}
+	else
+	{
+	    sru_message ("SR: process config changed for braille: brl_braille_transalation_table NULL!!!");
+	}
+
     }
     else
+    /*
     if (config->key && strcmp (config->key, BRAILLE_FILL_CHAR) == 0)
     {
     }
@@ -632,10 +667,7 @@ src_process_config_changed_for_braille (SRConfigStructure *config)
     {
 	if (config->newvalue)
 	{
-	    if (src_braille_position_sensor != *((gint*)config->newvalue))
-	    {
-		src_braille_position_sensor = *((gint*)config->newvalue);
-	    }
+	    src_braille_position_sensor = *((gint*)config->newvalue);
 	}
     }
     else
@@ -645,10 +677,7 @@ src_process_config_changed_for_braille (SRConfigStructure *config)
     {
 	if (config->newvalue)
 	{
-	    if (src_braille_optical_sensor != *((gint*)config->newvalue))
-	    {
-		src_braille_optical_sensor = *((gint*)config->newvalue);
-	    }
+	    src_braille_optical_sensor = *((gint*)config->newvalue);
 	}
     }
     
@@ -678,17 +707,6 @@ src_process_config_changed_for_srcore (SRConfigStructure *config)
 	    }
 	}
     }
-/*    else	
-    if (config->key && strcmp (config->key, SRCORE_FIND_TYPE) == 0)
-    {
-	sru_message ("Find Type event occured");
-    }
-    else
-    if (config->key && strcmp (config->key, SRCORE_FIND_TEXT) == 0)
-    {
-	sru_message ("Find Text event occured");
-    }
-*/    else
     if (config->key && 
 	config->type == CFGT_BOOL && 
 	strcmp (config->key, SRCORE_BRAILLE_ACTIVE) == 0)
@@ -698,7 +716,6 @@ src_process_config_changed_for_srcore (SRConfigStructure *config)
 	    if (src_use_braille != *((gboolean*)config->newvalue))
 	    {
 		src_use_braille = *((gboolean*)config->newvalue);
-	    
 		if (src_use_braille)
 		{
 		    src_use_braille = src_braille_init ();
@@ -786,10 +803,7 @@ src_process_config_changed_for_srcore (SRConfigStructure *config)
     {
 	if (config->newvalue)
 	{
-	    if (src_mouse_take != *((gboolean*)config->newvalue))
-	    {
-		src_mouse_take = *((gboolean*)config->newvalue);
-	    }
+	    src_mouse_take = *((gboolean*)config->newvalue);
 	}
     }
     else
@@ -799,10 +813,7 @@ src_process_config_changed_for_srcore (SRConfigStructure *config)
     {
 	if (config->newvalue)
 	{
-	    if (src_mouse_click != *((gboolean*)config->newvalue))
-	    {
-		src_mouse_click = *((gboolean*)config->newvalue);
-	    }
+	    src_mouse_click = *((gboolean*)config->newvalue);
 	}
     }
     else
@@ -824,7 +835,16 @@ src_process_config_changed_for_speech (SRConfigStructure *config)
     if (!src_use_speech)
 	return;
 
-    src_speech_proces_config_changed (config);
+    switch (config->module)
+    {
+	case CFGM_SPEECH:
+	case CFGM_SPEECH_VOICE:
+	    src_speech_process_config_changed (config);
+	    break;
+	default:
+	    src_speech_process_voice_config_changed (config);
+	    break;
+    }
 }
 
 static void
@@ -887,7 +907,7 @@ static gboolean src_crosswire	    	= FALSE;
 		{
 		    gchar *tmp;
 		    tmp = g_strdup_printf ("%s %s", 
-					   _("curent cursor is"), 
+					   _("current cursor is"), 
 					   cursors);
 		    src_say_message (tmp);
 		    g_free (tmp);
@@ -1034,8 +1054,8 @@ static gboolean src_crosswire	    	= FALSE;
 	    {
 		gchar *tmp;
 		tmp = g_strdup_printf (src_crosswire ? 
-				       _("crosswire on") :
-				       _("crosswire off"));
+				       _("crosshair on") :
+				       _("crosshair off"));
 		src_say_message (tmp);
 		g_free (tmp);
 	    }
@@ -1068,8 +1088,8 @@ static gboolean src_crosswire	    	= FALSE;
 	    {
 		gchar *tmp;
 		tmp = g_strdup_printf (crosswire_clip ? 
-				       _("crosswire clip on") :
-				       _("crosswire clipp off"));
+				       _("crosshair clip on") :
+				       _("crosshair clip off"));
 		src_say_message (tmp);
 		g_free (tmp);
 	    }
@@ -1108,7 +1128,7 @@ static gboolean src_crosswire	    	= FALSE;
 	    {
 		gchar *tmp;
 		tmp = g_strdup_printf ("%s %d", 
-				       _("crosswire size"), 
+				       _("crosshair size"), 
 				       crosswire_size);
 		src_say_message (tmp);
 		g_free (tmp);
@@ -1144,7 +1164,7 @@ static gboolean src_crosswire	    	= FALSE;
 	    {
 		gchar *tmp;
 		tmp = g_strdup_printf ("%s %u",
-				       _("crosswire color"),
+				       _("crosshair color"),
 				       crosswire_color);
 		src_say_message (tmp);
 		g_free (tmp);
@@ -1600,10 +1620,6 @@ src_process_config_changed_for_keyboard (SRConfigStructure *config)
 {
     if (!config) 
 	return;
-    if (config->newvalue)
-    {
-/*	ke_config_changed (config);*/
-    }
 }
 
 
@@ -1661,11 +1677,9 @@ src_process_config_changed (SREvent *event,
     	case CFGM_MAGNIFIER:
 	    src_process_config_changed_for_magnifier (config); 
 	    break;
-	case CFGM_SPEECH_VOICE_PARAM:
-	    src_speech_proces_voice_config_changed (config);
-	    break;
 	case CFGM_SPEECH:
 	case CFGM_SPEECH_VOICE:
+	case CFGM_SPEECH_VOICE_PARAM:
 	    src_process_config_changed_for_speech (config);
 	    break;
 	case CFGM_SRCORE:
@@ -1723,7 +1737,7 @@ src_kb_key_echo ()
 	return FALSE;
     message = src_xml_format ("TEXT", src_speech_get_voice ("system"), src_last_key_echo);
     if (message && message[0])
-	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_MESSAGE);
     g_free (message); 
     
     return TRUE;    
@@ -1788,9 +1802,9 @@ src_kb_cursor_echo ()
 	return FALSE;
     message = src_xml_format ("TEXT", src_speech_get_voice ("system"), src_last_cursor_echo);
     if (message && message[0])
-	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_SYSTEM);
     g_free (message); 
-    
+
     return TRUE;    
 }
 
@@ -2097,152 +2111,6 @@ src_translate_sensor_code (gchar *sensor_code,
     return rv;
 }
 
-SRLong src_text_index;
-
-static void
-src_position_sensor_action (gint index)
-{
-    if (!src_crt_sro)
-	return;
-	
-    src_text_index = index - 4 >= 0 ? index - 4 : 0;
-
-    switch (src_braille_position_sensor)
-    {
-	case 0:
-	    break;
-	case 1: 
-	    src_cmd_queue_add ("mouse goto current");
-	    src_cmd_queue_process ();
-	    break;
-	case 2:
-	    src_cmd_queue_add ("mouse goto current");
-	    src_cmd_queue_add ("mouse left click");
-	    src_cmd_queue_process ();
-	    break;
-	case 3:
-	    src_cmd_queue_add ("mouse goto current");
-	    src_cmd_queue_add ("mouse right click");
-	    src_cmd_queue_process ();
-	    break;
-	case 4:
-	    src_cmd_queue_add ("mouse goto current");
-/*
-	    src_cmd_queue_add ("mouse right click");
-	    src_cmd_queue_add ("mouse left click");
-*/
-	    src_cmd_queue_process ();
-	    break;
-	case 5:
-	    src_cmd_queue_add ("move caret");
-	    src_cmd_queue_process ();
-	    break;
-	case 6:
-	    src_cmd_queue_add ("show x coordinate");
-	    src_cmd_queue_process ();
-	    break;
-	case 7:
-	    src_cmd_queue_add ("show y coordinate");
-	    src_cmd_queue_process ();
-	    break;
-	case 8:
-	    src_cmd_queue_add ("font style");
-	    src_cmd_queue_process ();
-	    break;
-	case 9:
-	    src_cmd_queue_add ("font name");
-	    src_cmd_queue_process ();
-	    break;
-	case 10:
-	    src_cmd_queue_add ("font size");
-	    src_cmd_queue_process ();
-	    break;
-	case 11:
-	    sru_message ("sensor index 10 not implemented");
-	    break;
-	case 12:
-	    src_cmd_queue_add ("text attributes");
-	    src_cmd_queue_process ();
-	    break;
-	
-	default:
-	    sru_assert_not_reached ();
-	    break;
-    }	
-}
-
-static void
-src_optical_sensor_action (gint index)
-{
-    if (!src_crt_sro)
-	return;
-
-    src_text_index = index - 4 >= 0 ? index - 4 : 0;
-
-    switch (src_braille_optical_sensor)
-    {
-	case 0:
-	    break;
-	case 1: 
-	    src_cmd_queue_add ("mouse goto current");
-	    src_cmd_queue_process ();
-	    break;
-	case 2:
-	    src_cmd_queue_add ("mouse goto current");
-	    src_cmd_queue_add ("mouse left click");
-	    src_cmd_queue_process ();
-	    break;
-	case 3:
-	    src_cmd_queue_add ("mouse goto current");
-	    src_cmd_queue_add ("mouse right click");
-	    src_cmd_queue_process ();
-	    break;
-	case 4:
-	    src_cmd_queue_add ("mouse goto current");
-/*
-	    src_cmd_queue_add ("mouse right click");
-	    src_cmd_queue_add ("mouse left click");
-*/
-	    src_cmd_queue_process ();
-	    break;
-	case 5:
-	    src_cmd_queue_add ("move caret");
-	    src_cmd_queue_process ();
-	    break;
-	case 6:
-	    src_cmd_queue_add ("show x coordinate");
-	    src_cmd_queue_process ();
-	    break;
-	case 7:
-	    src_cmd_queue_add ("show y coordinate");
-	    src_cmd_queue_process ();
-	    break;
-	case 8:
-	    src_cmd_queue_add ("font style");
-	    src_cmd_queue_process ();
-	    break;
-	case 9:
-	    src_cmd_queue_add ("font name");
-	    src_cmd_queue_process ();
-	    break;
-	case 10:
-	    src_cmd_queue_add ("font size");
-	    src_cmd_queue_process ();
-	    break;
-	case 11:
-	    sru_message ("sensor index 10 not implemented");
-	    break;
-	case 12:
-	    src_cmd_queue_add ("text attributes");
-	    src_cmd_queue_process ();
-	    break;
-	default:
-	    sru_assert_not_reached ();
-	    break;
-    }	
-}
-
-
 static void
 src_process_braille_sensor (gchar *code)
 {
@@ -2257,10 +2125,10 @@ src_process_braille_sensor (gchar *code)
 	switch (type)
 	{
 	    case BST_PS:
-		src_position_sensor_action (index);
+		src_ctrl_position_sensor_action (index);
 		break;
 	    case BST_HOS:
-		src_optical_sensor_action (index);
+		src_ctrl_optical_sensor_action (index);
 		break;
 	    default:
 	    	fprintf (stderr, "SR: unsuported sensor code\n");
@@ -2281,7 +2149,8 @@ src_process_sro_event (SREvent *event,
     if (sre_get_event_data (event, (void**)&obj))
     {
 	src_process_sro (obj);
-	src_magnifier_start_panning (obj);
+	if (src_use_magnifier)
+	    src_magnifier_start_panning (obj);
     }
 /*
     {
@@ -2328,29 +2197,18 @@ src_process_mouse_event (SREvent *event,
 	
     if (sre_get_event_data (event, (void**)&point))
     {
-	static gboolean process = FALSE;
-	if (flags == 1) /* mouse:rel event */
-	{
-	    process = TRUE;
-	    src_magnifier_stop_panning ();
-	}
-	else if (process)
-	{
-	    gchar *magout;
-	    magout = g_strdup_printf ("<MAGOUT><ZOOMER ID =\"%s\" tracking=\"mouse\" ROILeft =\"%d\" ROITop =\"%d\" ROIWidth =\"%d\" ROIHeight=\"%d\"></ZOOMER></MAGOUT>",
+	gchar *magout;
 
+	src_magnifier_stop_panning ();
+	magout = g_strdup_printf ("<MAGOUT><ZOOMER ID =\"%s\" tracking=\"mouse\" ROILeft =\"%d\" ROITop =\"%d\" ROIWidth =\"%d\" ROIHeight=\"%d\"></ZOOMER></MAGOUT>",
 				"generic_zoomer",
 				point->x, point->y, point->x + 1, point->y + 1);
-	    if (magout)
-	    	src_magnifier_send (magout);
-	    g_free (magout);
-	    process = FALSE;
-	}
+	if (magout)
+	    src_magnifier_send (magout);
+	g_free (magout);
     }
 }
 
-
-
 static void
 src_event_sink(SREvent *event_, 
 	       unsigned long flags)
@@ -2399,6 +2257,15 @@ src_event_sink(SREvent *event_,
 		    break;		 
 	    }
 */	
+	    switch (type)
+	    {
+		case SR_EVENT_SRO:
+		case SR_EVENT_WINDOW:
+		    src_ctrl_flat_mode_terminate ();
+		    break;
+		default:
+		    break;
+	    }
 	    switch (type)
 	    {
 		case SR_EVENT_CONFIG_CHANGED:
@@ -2435,15 +2302,6 @@ src_event_sink(SREvent *event_,
 		    sru_assert_not_reached ();
 		    break;
 	    }
-	    switch (type)
-	    {
-		case SR_EVENT_SRO:
-		case SR_EVENT_WINDOW:
-		    src_ctrl_flat_mode_terminate ();
-		    break;
-		default:
-		    break;
-	    }
 	}
 	sre_release_reference (event);
     }
@@ -2484,24 +2342,48 @@ static void
 src_get_defaults ()
 {
     /* get braille port*/
+/*
     if (MIN_BRAILLE_PORT <= src_braille_port && src_braille_port <= MAX_BRAILLE_PORT)
+    {
 	SET_BRAILLE_CONFIG_DATA (BRAILLE_PORT_NO, CFGT_INT, &src_braille_port);
+    }
     else
     {
 	gint default_port = DEFAULT_BRAILLE_PORT_NO;
 	GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_PORT_NO, CFGT_INT, 
 		    &src_braille_port, &default_port);
     }
-    
+*/    
     /* get braille device*/
+/*
     if (src_braille_device)
+    {
 	SET_BRAILLE_CONFIG_DATA (BRAILLE_DEVICE, CFGT_STRING, &src_braille_device);
+    }
     else
     {
 	gchar *default_device = DEFAULT_BRAILLE_DEVICE;
 	GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_DEVICE, CFGT_STRING, 
 		&src_braille_device, default_device);
     }
+*/    
+    {
+	gchar *default_braille_transaltion_table = DEFAULT_BRAILLE_TRANSLATION;
+	GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_TRANSLATION, CFGT_STRING, 
+		&src_braille_translation_table, default_braille_transaltion_table);
+    }
+    
+    {
+	gchar *default_braille_style = DEFAULT_BRAILLE_STYLE;
+	GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_STYLE, CFGT_STRING, 
+		&src_braille_style, default_braille_style);
+    }
+
+    {
+	gchar *default_braille_cursor_style = DEFAULT_BRAILLE_CURSOR_STYLE;
+	GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_CURSOR_STYLE, CFGT_STRING, 
+		&src_braille_cursor_style, default_braille_cursor_style);
+    }
     
     /* get braille - position sensor value */
     {
@@ -2517,16 +2399,30 @@ src_get_defaults ()
 		    CFGT_INT, &src_braille_optical_sensor, &default_sensor);
     }
     
-
-    GET_SRCORE_CONFIG_DATA_WITH_DEFAULT (SRCORE_BRAILLE_ACTIVE, CFGT_BOOL, 
+    if ((gint)src_use_braille == -1)
+    {
+	GET_SRCORE_CONFIG_DATA_WITH_DEFAULT (SRCORE_BRAILLE_ACTIVE, CFGT_BOOL, 
 	    &src_use_braille, &src_use_braille);
-    GET_SRCORE_CONFIG_DATA_WITH_DEFAULT (SRCORE_BRAILLE_MONITOR_ACTIVE, CFGT_BOOL, 
+    }
+    
+    if ((gint)src_use_braille_monitor == -1)
+    {
+	GET_SRCORE_CONFIG_DATA_WITH_DEFAULT (SRCORE_BRAILLE_MONITOR_ACTIVE, CFGT_BOOL, 
 	    &src_use_braille_monitor, &src_use_braille_monitor);
-    GET_SRCORE_CONFIG_DATA_WITH_DEFAULT (SRCORE_SPEECH_ACTIVE, CFGT_BOOL, 
-	    &src_use_speech, &src_use_speech);
-    GET_SRCORE_CONFIG_DATA_WITH_DEFAULT (SRCORE_MAGNIF_ACTIVE, CFGT_BOOL, 
-	    &src_use_magnifier, &src_use_magnifier);
+    }
+    
+    if ((gint)src_use_speech == -1)
+    {
+	GET_SRCORE_CONFIG_DATA_WITH_DEFAULT (SRCORE_SPEECH_ACTIVE, CFGT_BOOL, 
+					    &src_use_speech, &src_use_speech);
+    }
 
+    if ((gint)src_use_magnifier == -1)
+    {
+	GET_SRCORE_CONFIG_DATA_WITH_DEFAULT (SRCORE_MAGNIF_ACTIVE, CFGT_BOOL, 
+					    &src_use_magnifier, &src_use_magnifier);
+    }
+    
     GET_SRCORE_CONFIG_DATA_WITH_DEFAULT (KEYBOARD_TAKE_MOUSE, CFGT_BOOL, 
 	    &src_mouse_take, &src_mouse_take);
     GET_SRCORE_CONFIG_DATA_WITH_DEFAULT (KEYBOARD_SIMULATE_CLICK, CFGT_BOOL, 
@@ -2534,9 +2430,10 @@ src_get_defaults ()
 	    
 }
 
-gchar *src_message;
+static gchar *src_message;
+
 gboolean 
-src_say_message (gchar *message)
+src_say_message (const gchar *message)
 {
     if (src_message)
 	g_free (src_message);
@@ -2567,6 +2464,7 @@ src_init()
     SRLClient client;
     gchar *message = _("Welcome to Gnopernicus");
     gboolean brl_error = FALSE;
+    gboolean initialize_finished = FALSE;
 
     /* initialize SR Utils */
     sru_init();
@@ -2578,7 +2476,6 @@ src_init()
     src_ctrl_init ();
     
 /*    src_speech_count_mode = DEFAULT_SPEECH_REPEAT_TYPE;*/
-
     src_get_defaults ();
 
     /* initialize Braille */
@@ -2588,7 +2485,6 @@ src_init()
 	if (!src_use_braille)
 	    brl_error = TRUE;
     }
-	
     if (src_use_braille_monitor)
 	src_use_braille_monitor = src_braille_monitor_init ();
  	
@@ -2639,14 +2535,18 @@ src_init()
     src_last_cursor_echo 	= NULL;
     if (brl_error)
 	src_say_message (_("braille device can not be initialized"));
-    
+
+    /* src_init finished */
+    SET_SRCORE_CONFIG_DATA(SRCORE_INITIALIZATION_END, CFGT_BOOL, &initialize_finished);
+    initialize_finished = !initialize_finished;
+    SET_SRCORE_CONFIG_DATA(SRCORE_INITIALIZATION_END, CFGT_BOOL, &initialize_finished);    
 }
 
 static void
 src_terminate()
 {
     gboolean exitack = TRUE;
-    gchar *message = _("now gnopernicus will gonna exit");
+    gchar *message = _("Gnopernicus is now exiting. Have a nice day!");
 
     if (src_focused_sro)
 	sro_release_reference (src_focused_sro);
@@ -2665,6 +2565,11 @@ src_terminate()
 	
     if (src_use_speech)
 	src_say_message (message);
+    /* FIXME remus: maybe this function should not be called here, in respect to
+    current design */
+    /* terminate screen review in case that Exit was called and gnopernicus
+    is still in flat review mode*/
+    screen_review_terminate ();
 	
     /* terminate SRLow */
     srl_remove_client (src_srl_client);
@@ -2673,6 +2578,10 @@ src_terminate()
     /* terminate Keyboard */
     ke_terminate();
 
+    /* terminate Magnifier */
+    if (src_use_magnifier)
+	src_magnifier_terminate (); 
+
     /* terminate Braille */
     if (src_use_braille)
 	src_braille_terminate ();
@@ -2681,10 +2590,6 @@ src_terminate()
     if (src_use_braille_monitor)
 	src_brlmon_terminate ();
 
-    /* terminate Magnifier */
-    if (src_use_magnifier)
-	src_magnifier_terminate (); 
-
     /* terminate Speech */
     if (src_use_speech)
 	src_speech_terminate ();
@@ -2720,14 +2625,113 @@ gint
 main (gint argc, 
       gchar **argv)
 {
-    gchar *device = NULL;
+    gboolean enable_braille 	= FALSE;
+    gboolean disable_braille 	= FALSE;
+    gboolean enable_speech 	= FALSE;
+    gboolean disable_speech 	= FALSE;
+    gboolean enable_magnifier	= FALSE;
+    gboolean disable_magnifier	= FALSE;
+    gboolean enable_braille_monitor 	= FALSE;
+    gboolean disable_braille_monitor 	= FALSE;    
 
     struct poptOption poptopt[] = 
     {		
-	{"device", 	'd', POPT_ARG_STRING, 	&device,  	'd', "Braille Device", 		"device_name"},
-	{"port", 	'p', POPT_ARG_INT, 	&src_braille_port,'p', "Serial port (ttyS)", 	"serial_port_no"},
-	{NULL, 		0,0, NULL, 0}
+	{
+	 "enable-braille",	
+	 'b', 	
+	 POPT_ARG_NONE, 
+	 &enable_braille,		
+	 0, 
+	 "enable braille service", 
+	 NULL
+	},
+	{
+	 "disable-braille",	
+	 'r', 	
+	 POPT_ARG_NONE, 
+	 &disable_braille,		
+	 0, 
+	 "enable braille service", 
+	 NULL
+	},
+	{
+	 "enable-speech",	
+	 's', 	
+	 POPT_ARG_NONE, 
+	 &enable_speech,		
+	 0, 
+	 "enable speech service", 
+	 NULL
+	},
+	{
+	 "disable-speech",	
+	 'e', 	
+	 POPT_ARG_NONE, 
+	 &disable_speech,		
+	 0, 
+	 "disable speech service", 
+	 NULL
+	},
+	{
+	 "enable-magnifier",	
+	 'm', 	
+	 POPT_ARG_NONE, 
+	 &enable_magnifier,		
+	 0, 
+	 "enable magnifier service",  
+	 NULL
+	},
+	{
+	 "disable-magnifier",	
+	 'a', 	
+	 POPT_ARG_NONE, 
+	 &disable_magnifier,		
+	 0, 
+	 "disable magnifier service",  
+	 NULL
+	},
+
+	{
+	 "enable-braille-monitor", 
+	 'o', 
+	 POPT_ARG_NONE, 
+	 &enable_braille_monitor, 	
+	 0, 
+	 "enable braille monitor service", 
+	 NULL
+	},
+	{
+	 "disable-braille-monitor", 
+	 'n', 
+	 POPT_ARG_NONE, 
+	 &disable_braille_monitor, 	
+	 0, 
+	 "disable braille monitor service", 
+	 NULL
+	},
+	{
+	 "braille-port", 
+	 'p',   
+	 POPT_ARG_INT,  
+	 &src_braille_port, 	
+	 0, 
+	 "Serial port (ttyS)", 
+	 "ttyS[1..4]"
+	},
+	{
+	 "braille-device", 
+	 'e', 
+	 POPT_ARG_STRING, 
+	 &src_braille_device, 	
+	 0, 
+	 "Braille Device", 
+	 "DEVICE NAME"
+	},
+	{
+	 NULL, 0,     0, NULL, 0
+	}
     };
+    
 #ifdef ENABLE_NLS
     bindtextdomain (GETTEXT_PACKAGE, GNOPERNICUSLOCALEDIR);
     textdomain (GETTEXT_PACKAGE);
@@ -2739,8 +2743,7 @@ main (gint argc,
     fprintf (stderr, "\n**********************");
     fprintf (stderr, "\n* SCREEN READER CORE *");
     fprintf (stderr, "\n**********************\n\n");
-    
-    src_braille_device = NULL;
+        
     gnome_program_init ("srcore", VERSION,
 		    LIBGNOMEUI_MODULE,
 		    argc, argv,
@@ -2749,17 +2752,24 @@ main (gint argc,
 		    LIBGNOMEUI_PARAM_CRASH_DIALOG, FALSE,
 		    NULL);
     sru_log_init ();
-	
-    if (device)
-	{
-	    if (src_braille_device) g_free (src_braille_device);
-	    src_braille_device = g_strdup (device);
-	    g_free (device);
-	}
-    if (src_braille_port <= MIN_BRAILLE_PORT && src_braille_port >= MAX_BRAILLE_PORT)
-	{
-	    src_braille_port = DEFAULT_BRAILLE_PORT_NO;
-	}
+
+    if (src_braille_port <= MIN_BRAILLE_PORT && 
+	src_braille_port >= MAX_BRAILLE_PORT)
+    {
+	src_braille_port = DEFAULT_BRAILLE_PORT_NO;
+    }
+
+    src_use_braille = (disable_braille ? FALSE :
+			(enable_braille ? TRUE : -1));
+
+    src_use_braille_monitor = (disable_braille_monitor ? FALSE :
+			      (enable_braille_monitor ? TRUE : -1));
+    
+    src_use_speech = (disable_speech ? FALSE :
+		     (enable_speech ? TRUE : -1));
+
+    src_use_magnifier = (disable_magnifier ? FALSE :
+			(enable_magnifier  ? TRUE : -1));
 
     src_init ();
 
@@ -2768,6 +2778,7 @@ main (gint argc,
     src_terminate ();
     
     sru_log_terminate ();
+    
     return EXIT_SUCCESS;
 }
 
