@@ -1307,6 +1307,7 @@ static struct
         {"frame",			"FRM",	"FRAME",		"Frame",		},
         {"glass-pane",			"GPN",	"GLASS PANE",		"Glass pane",		},
 	{"HTML-container",		"HTM",	"HTML CONTAINER",	"H T M L container",	},
+	{"hyper link",			"LNK",	"LINK",			"link",		},
         {"icon",			"ICO",	"ICON",			"Icon",			},
         {"internal-frame",		"IFR",	"INTERNAL FRAME",	"Internal frame",	},
 	{"label",			"LBL",	"LABEL",		"Label",		},
@@ -1390,7 +1391,10 @@ src_presentation_get_role_name_for_braille (const gchar *role)
 
     index = src_presentation_get_role_index (role);
 
+    if (index)
     return src_presentation_role_pres[index].braille_short;
+    else
+	return role;
 }
 
 static G_CONST_RETURN gchar* 
@@ -1402,7 +1406,10 @@ src_presentation_get_role_name_for_speech (const gchar *role)
 
     index = src_presentation_get_role_index (role);
 
+    if (index)
     return src_presentation_role_pres[index].speech_short;
+    else
+	return role;
 }
 
 
